#include <SoftwareSerial.h>                      //include serial library
SoftwareSerial bluetooth(2, 3);                  //TX and RX

#include <Servo.h>                               //inclue servo library
//Servo steerServo;                                //call the servo steerServo
Servo plantServo;                                //call the servo plantServo

#include <dht.h>                                 //include DHT library
dht DHT;                                         //call the dht DHT

const int ENA = 10;                              //left motor
const int IN1 = 9;
const int IN2 = 8;
const int IN3 = 7;                               //right motor
const int IN4 = 6;
const int ENB = 5;
int motorSpeed = 75;

int humiTempPin = A0;

int amountOfSeeds;

int turnAngle = 90;
boolean lastTimeLeft = true;

const int trigPin = 12;
const int echoPin = A4;
String distString;
String tempString;
String humiString;

int opened = 0;
int closed = 30;
int seedTimer = 1000;
long duration;
float distance;

String buff;

void setup() {
  Serial.begin(9600);
  bluetooth.begin(9600);

  pinMode(13, OUTPUT);
  digitalWrite(13, HIGH);

  for (int i = 5; i <= 10; i++) {                //all motor pins
    pinMode(i, OUTPUT);
  }
  pinMode(trigPin, OUTPUT);                      //dist sensor pins
  pinMode(echoPin, INPUT);

  analogWrite(ENA, motorSpeed);                  //set speed to 50
  analogWrite(ENB, motorSpeed);                  //on both motors
  // steerServo.attach(11);                         //servo on pin 11
  // steerServo.write(90);                          //set the wheels straight
  plantServo.attach(4);                          //servo on pin 4
  plantServo.write(closed);                      //close the container
  buff = "";
  pinMode(A4, OUTPUT);
  digitalWrite(A4, HIGH);
  pinMode(A1, INPUT);

}



void loop() {
  analogWrite(ENA, motorSpeed);                  //adjust motorspeed
  analogWrite(ENB, motorSpeed);                  //if neccesary

  //    digitalWrite(trigPin, HIGH);
  //    delayMicroseconds(10);                         //send a pulse to dist sensor
  //    digitalWrite(trigPin, LOW);
  //    duration = pulseIn(echoPin, HIGH);             //read the incoming pulse, so the traveltime of sound
  //    distance = duration * 0.034 / 2;               //calculate the distance
  //
  //    distString = "";                               //clear the string
  //    distString += "D";                             //add D to the string
  //    distString += String(distance);                //add the value
  //    distString += ",";                             //add a ,
  //    bluetooth.print(distString);                   //send the distance
  //    if (distance < 10) {
  //    //  bluetooth.print("A,");
  //    }

  //  int chk = DHT.read11(humiTempPin);             //read all the data
  //  delay(50);

  //  tempString = "";                               //clear the string
  //  tempString += "T";                             //add a T
  //  tempString += String(DHT.temperature);         //add the value
  //  tempString += ",";                             //add a ,
  //  bluetooth.print(String(tempString));           //send it
  //
  //  humiString = "";                               //clear the string
  //  humiString += "H";                             //add an H
  //  humiString += String(DHT.temperature);         //add the value
  //  humiString += ",";                             //add a ,
  //  bluetooth.print(humiString);                   //send it

  checkBT();                                     //check if there is BT info
}

void checkBT() {
  while (bluetooth.available() > 0) {            //if there is data
    SerialEvent(char(bluetooth.read()));         //read the data
  }
}


void SerialEvent(char input) {

  if (input != ',') {
    buff += input;
  } else {
    Serial.println(buff);

    switch (buff.charAt(0)) {                       //decide on what to do based on BT input

      //                   *****MANUAL INPUT*****

      case '0':                                     //STOP DRIVING
        halt();
        break;

      case '1':                                     //DRIVE FORWARD
        forward();
        break;

      case '2':                                     //DRIVE BACKWARD
        backward();
        break;

      case 'L':                                     //TURN LEFT
        turn(120);
        break;

      case 'R':                                    //TURN RIGHT
        turn(60);
        break;

      case 'D':                                    //DROP A SEED
        dropSeed();
        break;

      case 'V':                                    //SET THE SPEED
        buff.remove(0, 1);                         //remove the command
        motorSpeed = buff.toInt();                 //stor the int
        break;

      //                   *****AUTOMATIC INPUT*****

      case 'S':                                    //MAKE A LINE OF SEEDS
        buff.remove(0, 1);                         //remove the command
        amountOfSeeds = buff.toInt();              //store the int
        plant(amountOfSeeds);                      //start planting
        break;

      case 'T':                                    //MAKE A U-TURN
        uTurn();
        break;

      case 'A':                                    //SET THE TIME BETWEEN SEEDS
        buff.remove(0, 1);                         //remove the command
        seedTimer = buff.toInt();                  //store the int
        break;

    }
    buff = "";
  }
}


void halt() {
  digitalWrite(IN1, LOW);                        //no power
  digitalWrite(IN2, LOW);                        //to motors
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, LOW);
  // steerServo.write(90);                             //reset the servo
}


void forward() {
  digitalWrite(IN1, HIGH);                       //make the motors
  digitalWrite(IN2, LOW);                        //spin in 1 direction
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
}


void backward() {
  digitalWrite(IN1, LOW);                        //make the motors
  digitalWrite(IN2, HIGH);                       //spin in other direction
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH);
}

void dropSeed() {
  plantServo.write(opened);
  delay(135);                                    //open the container
  plantServo.write(closed);
  bluetooth.write("S,");
  Serial.write("seed has been dropped");
}

void turn(int turnAngle) {
  //  steerServo.write(turnAngle);                      //turn the wheels
  forward();                                     //drive
}

void plant(int amountOfSeeds) {
  dropSeed();                                    //drop a seed
  for (int i = 1; i < amountOfSeeds; i++) {     //do this "amountOfSeeds" times:
    checkBT();                                   //check if there is BT info
    if (buff.charAt(0) == '0') {
      return;
    }
    forward();                                   //drive
    delay(seedTimer * 1000);                          //for some time
    halt();                                      //stop
    dropSeed();                                  //drop a seed
    delay(500);
  }
  bluetooth.write("F,");                          //request new command
}

void uTurn() {
  if (lastTimeLeft) {                            //if car turned left
    // steerServo.write(60);                        //turn wheels right
  }
  else {                                         //if car turned right
    //  steerServo.write(120);                       //turn wheels left
  }
  forward();                                   //drive
  delay(1000);                                 //for 1 sec
  halt();                                      //stop
  lastTimeLeft = !lastTimeLeft;                //flip boolean

  bluetooth.write("E,");
}
