package com.example.smartenvironmentsapp;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.BlurMaskFilter;
import android.graphics.Color;
import android.graphics.Paint;
import android.inputmethodservice.KeyboardView;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import com.example.smartenvironmentsapp.views.VisAutomatic;
import com.google.android.material.slider.Slider;

import java.util.Timer;

public class AutomaticFragment extends Fragment {

    View v;
    Slider widthSlider;
    Slider heightSlider;
    VisAutomatic visualAutomatic;
    Button startStopButton;
    boolean started;
    int pathWidth;
    int pathHeight;
    int seedsDropped;
    int rowsCompleted;
    CountDownTimer timer;

    private FragmentAutomaticListener listener;

    public interface FragmentAutomaticListener {
        int getSecondsBetweenSeeds();
        void SendBluetooth(String input);
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (context instanceof FragmentAutomaticListener) {
            listener = (FragmentAutomaticListener) context;
        } else {
            throw new RuntimeException(context.toString() + " must implement FragmentBluetoothListener");
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.automatic_fragment,container,false);

        visualAutomatic = v.findViewById(R.id.visAutomatic);
        widthSlider = v.findViewById(R.id.widthSlider);
        heightSlider = v.findViewById(R.id.heightSlider);
        startStopButton = v.findViewById(R.id.Start_Stop_Button);

        widthSlider.addOnChangeListener(new Slider.OnChangeListener() {
            @Override
            public void onValueChange(@NonNull Slider slider, float value, boolean fromUser) {
                if(!fromUser || started) return;
                visualAutomatic.generatePath((int) widthSlider.getValue(), (int) heightSlider.getValue());
            }
        });

        heightSlider.addOnChangeListener(new Slider.OnChangeListener() {
            @Override
            public void onValueChange(@NonNull Slider slider, float value, boolean fromUser) {
                if(!fromUser || started) return;
                visualAutomatic.generatePath((int) widthSlider.getValue(), (int) heightSlider.getValue());
            }
        });

        startStopButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if(started){
                    //Stop
                    started = false;
                    startStopButton.setText(R.string.Start);
                    visualAutomatic.reset();
                    if(timer != null) timer.cancel();


                } else{
                    //start
                    started = true;
                    startStopButton.setText(R.string.Stop);

                    pathWidth = (int) widthSlider.getValue();
                    pathHeight = (int) heightSlider.getValue();
                    seedsDropped = 0;
                    rowsCompleted = 0;

                    listener.SendBluetooth("A" + listener.getSecondsBetweenSeeds() + ",");
                    listener.SendBluetooth("S" + pathWidth + ",");
                }
            }
        });


        return v;
    }

    public void seedDropped(){
        if(!started) return;
        seedsDropped++;
        if(timer != null) timer.cancel();

        if(seedsDropped<=pathWidth) {
            timer = new CountDownTimer(listener.getSecondsBetweenSeeds() * 1000, 100) { //timer per between every seed drop
                @Override
                public void onTick(long millisUntilFinished) {
                    visualAutomatic.updateProgress(seedsDropped, (int) (listener.getSecondsBetweenSeeds() * 1000 - millisUntilFinished) / (listener.getSecondsBetweenSeeds() * 10), rowsCompleted,100);
                }

                @Override
                public void onFinish() {

                }
            }.start();
        }

    }

    public void rowCompleted(){
        rowsCompleted++;
        if(rowsCompleted<pathHeight){
            seedsDropped = 0;
            if(timer != null) timer.cancel();

            listener.SendBluetooth("T,");
            timer = new CountDownTimer(listener.getSecondsBetweenSeeds()*1000,100) {
                @Override
                public void onTick(long millisUntilFinished) {
                    visualAutomatic.updateProgress(seedsDropped, 100, rowsCompleted,(int) (listener.getSecondsBetweenSeeds() * 1000 - millisUntilFinished) / (listener.getSecondsBetweenSeeds() * 10));
//                    System.out.println((listener.getSecondsBetweenSeeds() * 10000 - millisUntilFinished) / (listener.getSecondsBetweenSeeds() * 100));
                }

                @Override
                public void onFinish() {

                }
            }.start();
        } else{
            Toast.makeText(getContext(),"Done",Toast.LENGTH_SHORT).show();
        }
    }

}
