package com.example.smartenvironmentsapp;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;

import java.io.IOException;
import java.io.OutputStream;
import java.util.UUID;

import static android.provider.AlarmClock.EXTRA_MESSAGE;

public class BluetoothFragment extends Fragment {
    ListView LV_BtDevices;
    BluetoothAdapter btAdaptor;
    BluetoothDevice connectedDevice;
    String[] btDevices;
    String[] btDevicesAddress;
    BluetoothSocket btSocket;
    static final UUID mUUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    OutputStream outputStream;
    View v;
    private FragmentBluetoothListener listener;

    public interface FragmentBluetoothListener {
        void sendBluetoothData(BluetoothAdapter btAdaptor, BluetoothDevice connectedDevice, BluetoothSocket btSocket, OutputStream outputStream);
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.bluetooth_fragment, container, false);

        //List_btDevices
        btAdaptor = BluetoothAdapter.getDefaultAdapter(); //set's the bluetooth adaptor
        String devicesStr = "";
        String devicesAddressStr = "";
        for (int i = 0; i < btAdaptor.getBondedDevices().toArray().length; i++) {
            BluetoothDevice[] bluetoothDevices = btAdaptor.getBondedDevices().toArray(new BluetoothDevice[0]);
            devicesStr += bluetoothDevices[i].getName() + ",";
            devicesAddressStr += bluetoothDevices[i].getAddress() + ",";
        }
        System.out.println(devicesStr);
        btDevices = devicesStr.split(",");
        btDevicesAddress = devicesAddressStr.split(",");


        //set up the list view
        LV_BtDevices = v.findViewById(R.id.List_btDevices);
        ArrayAdapter<String> btDevices_Adapter = new ArrayAdapter<>(getContext(), android.R.layout.simple_list_item_1, btDevices);// sets the bluetooth devices in an adaptor
        LV_BtDevices.setAdapter(btDevices_Adapter); //add the bluetooth devices to the list view
        //onclick function of the list view.
        LV_BtDevices.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

                try {
                    if(btSocket != null) btSocket.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }


                Toast.makeText(getContext(), "Connecting...", Toast.LENGTH_SHORT).show();
                connectedDevice = btAdaptor.getRemoteDevice(btDevicesAddress[position]);

                if (connectBluetooth()) {
                    listener.sendBluetoothData(btAdaptor, connectedDevice, btSocket, outputStream);
                } else {
                    Toast.makeText(getContext(),"Couldn't connect to " + connectedDevice.getName(),Toast.LENGTH_SHORT).show();
                }

            }
        });

        return v;
    }

    private boolean connectBluetooth() {
        int counter = 0;
        do {
            try {
                btSocket = connectedDevice.createRfcommSocketToServiceRecord(mUUID);
                btSocket.connect();
                System.out.println(btSocket.isConnected());
            } catch (IOException e) {
                e.printStackTrace();
                return false;
            }
            counter++;
        } while (!btSocket.isConnected() && counter < 3);
        if (counter >= 3) {//if it couldn't connect, even after 3 tries.
            System.out.println("Failed to connect to: " + connectedDevice.getName());
            Toast.makeText(getContext(), "Failed to connect to: " + connectedDevice.getName(), Toast.LENGTH_SHORT).show();
            return false;// needs to return back
        }

        if (btSocket.isConnected()) {
            //the device successfully connected
        } else { //if something was wrong
            System.out.println("Failed to connect to: " + connectedDevice.getName());
            Toast.makeText(getContext(), "Failed to connect to: " + connectedDevice.getName(), Toast.LENGTH_SHORT).show();
            return false; // needs to return back
        }

        try {
            outputStream = btSocket.getOutputStream();
        } catch (IOException e) {
            e.printStackTrace();
            Toast.makeText(getContext(), "lost connection with: " + connectedDevice.getName(), Toast.LENGTH_SHORT).show();
            return false;
        }
        return true;
    }


    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (context instanceof FragmentBluetoothListener) {
            listener = (FragmentBluetoothListener) context;
        } else {
            throw new RuntimeException(context.toString() + " must implement FragmentBluetoothListener");
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
        listener = null;
    }
}
