package com.example.smartenvironmentsapp;

import androidx.annotation.NonNull;
import androidx.annotation.StyleableRes;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.material.bottomnavigation.BottomNavigationView;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import static com.example.smartenvironmentsapp.BluetoothFragment.mUUID;

public class MainActivity extends AppCompatActivity implements BluetoothFragment.FragmentBluetoothListener,ManualFragment.FragmentManualListener,AutomaticFragment.FragmentAutomaticListener {
    //fragments
    BluetoothFragment bluetoothFragment;
    AutomaticFragment automaticFragment;
    ManualFragment manualFragment;
    DashboardFragment dashboardFragment;

    //bluetooth
    BluetoothAdapter btAdaptor;
    BluetoothDevice connectedDevice;
    BluetoothSocket btSocket;
    OutputStream outputStream;
    //bluetooth receive data:
    InputStream inputStream;
    String buffer;
    Handler handler = new Handler();
    Runnable runnable;
    static final int delay = 100; //delay between checking for bluetooth messages


    //elements
    BottomNavigationView bottomNav;

    //things
    static final String NAME_ROBOT = "Treevie";

    //setting values
    int secondsBetweenSeeds = 2;

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater menuInflater = getMenuInflater();
        menuInflater.inflate(R.menu.menu,menu);
        return true;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        bottomNav = findViewById(R.id.bottom_navigation);
        bottomNav.setOnNavigationItemSelectedListener(navListener);
        bottomNav.setVisibility(View.GONE);//hides the element.

        bluetoothFragment = new BluetoothFragment();
        getSupportFragmentManager().beginTransaction().replace(R.id.fragmentContainer, bluetoothFragment).commit();

        btAdaptor = BluetoothAdapter.getDefaultAdapter();
        connectedDevice = btAdaptor.getRemoteDevice("98:D3:31:F5:D3:B0");
        try {
            btSocket = connectedDevice.createRfcommSocketToServiceRecord(mUUID);
            btSocket.connect();
            outputStream = btSocket.getOutputStream();
            sendBluetoothData(btAdaptor,connectedDevice,btSocket,outputStream);
        }catch (Exception e){
            System.out.println(e);
        }

        buffer = ""; //set's the buffer to an empty string
    }

    @Override
    public void sendBluetoothData(BluetoothAdapter btAdaptor, BluetoothDevice connectedDevice, BluetoothSocket btSocket, OutputStream outputStream) {
        this.btAdaptor = btAdaptor;
        this.connectedDevice = connectedDevice;
        this.btSocket = btSocket;
        this.outputStream = outputStream;
        try {
            inputStream = btSocket.getInputStream();
        }catch (Exception e){
            System.out.println("ERROR: " + e);
        }

        Toast.makeText(this, "Connected to " + connectedDevice.getName(),Toast.LENGTH_SHORT).show();

        manualFragment = new ManualFragment();
        automaticFragment = new AutomaticFragment();
        dashboardFragment = new DashboardFragment();

        bottomNav.setVisibility(View.VISIBLE);//Shows the element.
        getSupportFragmentManager().beginTransaction().replace(R.id.fragmentContainer, manualFragment).commit();
    }

    //loop:
    @Override
    protected void onResume() {
        handler.postDelayed(runnable = new Runnable() {
            @Override
            public void run() {
                handler.postDelayed(runnable,delay);
                //-------------------- Loop --------------------\\
                try {
                    while(inputStream.available()>0){
//                        inputStream.read(buffer);
                        char input = (char) inputStream.read();
                        if(input != ','){
                            buffer += input;
                        } else{
                            System.out.println(buffer);
                            receiveBluetooth(buffer);
                            buffer = "";
                        }
                    }
                }catch (Exception e){
                    System.out.println("---------------------------- NO Valid data ----------------------------");
                    System.out.println("ERROR: " + e);
                }
            }
        }, delay);
        super.onResume();
    }

    @Override
    protected void onPause() {
        handler.removeCallbacks(runnable); //stop handler when activity not visible
        super.onPause();
    }


    @Override
    public void SendBluetooth(String input) {
        byte[] sendData = input.getBytes();
        try {
            outputStream.write(sendData);
            
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //what to do when a bluetooth message has been received
    private void receiveBluetooth(String input){
        switch (input){
            case "A": //receive when the robot encounters a obstacle
                Toast.makeText(this,"ALERT: " + NAME_ROBOT + " is blocked", Toast.LENGTH_LONG).show();
                break;

            case "S"://when a seed has been dropped
                automaticFragment.seedDropped();
                break;
            case "F"://when the robot has finished a row/stroke
                automaticFragment.rowCompleted();
                break;
            case "E"://when the robot has finished the turn.
                SendBluetooth("S" + automaticFragment.pathWidth + ",");
                break;
        }
    }

    private BottomNavigationView.OnNavigationItemSelectedListener navListener = new BottomNavigationView.OnNavigationItemSelectedListener() {
        @Override
        public boolean onNavigationItemSelected(@NonNull MenuItem item) {
            Fragment selectedFragment = null;

            switch (item.getItemId()){
                case R.id.nav_manual:
                    selectedFragment = manualFragment;
                    break;
                case R.id.nav_auto:
                    selectedFragment = automaticFragment;
                    break;
                case R.id.nav_dashboard:
                    selectedFragment = dashboardFragment;
                    break;
            }
            getSupportFragmentManager().beginTransaction().replace(R.id.fragmentContainer,selectedFragment).commit();
            return true;
        }
    };

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()){
            case R.id.itemDisconnect:
                Toast.makeText(this, "Disconnected", Toast.LENGTH_SHORT).show();

                try {
                    if(btSocket != null) btSocket.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                bottomNav.setVisibility(View.GONE);//hides the bottom navigation bar.
                bluetoothFragment = new BluetoothFragment();
                getSupportFragmentManager().beginTransaction().replace(R.id.fragmentContainer, bluetoothFragment).commit();

                return true;
            case R.id.itemSettings:
                Toast.makeText(this, "Settings", Toast.LENGTH_SHORT).show();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public int getSecondsBetweenSeeds() {
        return secondsBetweenSeeds;
    }
}