package com.example.smartenvironmentsapp;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.content.res.Configuration;
import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatDelegate;
import androidx.constraintlayout.widget.ConstraintsChangedListener;
import androidx.fragment.app.Fragment;

import com.google.android.material.slider.Slider;

import java.io.OutputStream;

public class ManualFragment extends Fragment {

    //buttons
    ImageButton buttonUp;
    ImageButton buttonDown;
    ImageButton buttonLeft;
    ImageButton buttonRight;
    ImageButton buttonSeed;
    Slider speedSlider;
    View v;

    private FragmentManualListener listener;

    public interface FragmentManualListener {
        void SendBluetooth(String input);
    }

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        if (context instanceof FragmentManualListener) {
            listener = (FragmentManualListener) context;
        } else {
            throw new RuntimeException(context.toString() + " must implement FragmentBluetoothListener");
        }
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        v = inflater.inflate(R.layout.manual_fragment,container,false);

        //configure Buttons:
        buttonUp = v.findViewById(R.id.buttonUp);
        buttonDown = v.findViewById(R.id.buttonDown);
        buttonLeft = v.findViewById(R.id.buttonLeft);
        buttonRight = v.findViewById(R.id.buttonRight);
        buttonSeed = v.findViewById(R.id.button_DropSeed);
        speedSlider = v.findViewById(R.id.speedSlider);

//        buttonUp.setOnClickListener(this);
        buttonUp.setOnTouchListener(this::onTouch);
        buttonDown.setOnTouchListener(this::onTouch);
        buttonLeft.setOnTouchListener(this::onTouch);
        buttonRight.setOnTouchListener(this::onTouch);
        buttonSeed.setOnTouchListener(this::onTouch);

//        speedSlider.addOnChangeListener(sliderListener);
        speedSlider.setOnTouchListener(sliderListener);
        return v;
    }

    View.OnTouchListener sliderListener = new View.OnTouchListener() {
        @Override
        public boolean onTouch(View v, MotionEvent event) {
            Slider slider = (Slider) v;
            if(event.getAction() == MotionEvent.ACTION_UP){
                listener.SendBluetooth("V" + (int) slider.getValue() + ",");
            }
            return false;
        }
    };

//    @Override
    public boolean onTouch(View v, MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            // Pressed
//            int color = getResources().getColor(R.color.main); not supported anymore
            int color = getContext().getColor(R.color.main);
            switch (v.getId()){
                case R.id.buttonUp:
                    listener.SendBluetooth("1,");
                    buttonUp.setColorFilter(color);
                    break;
                case R.id.buttonDown:
                    listener.SendBluetooth("2,");
                    buttonDown.setColorFilter(color);
                    break;
                case R.id.buttonLeft:
                    listener.SendBluetooth("L,");
                    buttonLeft.setColorFilter(color);
                    break;
                case R.id.buttonRight:
                    listener.SendBluetooth("R,");
                    buttonRight.setColorFilter(color);
                    break;
                case R.id.button_DropSeed:
                    listener.SendBluetooth("D,");
                    buttonSeed.setBackgroundColor(getContext().getColor(R.color.SelcetedTransparent));
                    break;
            }
        } else if (event.getAction() == MotionEvent.ACTION_UP) {
            // Released

            //test for night mode and change the colors of the buttons.
            int nightModeFlags = getContext().getResources().getConfiguration().uiMode & Configuration.UI_MODE_NIGHT_MASK;
            int color;
            if(nightModeFlags == Configuration.UI_MODE_NIGHT_YES) color = Color.LTGRAY;
            else color = Color.GRAY;

            switch (v.getId()){
                case R.id.buttonUp:
                case R.id.buttonLeft:
                case R.id.buttonDown:
                case R.id.buttonRight:
                    listener.SendBluetooth("0,");

                    buttonUp.setColorFilter(color);
                    buttonDown.setColorFilter(color);
                    buttonRight.setColorFilter(color);
                    buttonLeft.setColorFilter(color);
                    break;
                case R.id.button_DropSeed:
                    buttonSeed.setBackgroundColor(getContext().getColor(R.color.transparent));
                    break;
            }
        }
        return true;
    }
}
