package com.example.smartenvironmentsapp.views;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.view.View;

import androidx.annotation.Nullable;

import com.example.smartenvironmentsapp.R;

public class VisAutomatic extends View {

    Rect baseRectangles[];
    Rect cornerRectangles[];
    Rect progressRectangles[];
    Rect progressCornerRectangles[];
    boolean circles[];
    Paint unfinishedPath_Paint;
    Paint unfinishedCircle_Paint;
    Paint finished_Paint;
    static  final int MARGIN = 50;
    int circleRadius;
    int rectHeight;
    int height;
    int width;
    int pathWidth;
    int pathHeight;

    public VisAutomatic(Context context) {
        super(context);
        init(null);
    }

    public VisAutomatic(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
        init(attrs);
    }

    public VisAutomatic(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(attrs);
    }

    public VisAutomatic(Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init(attrs);
    }

    private void init(@Nullable AttributeSet set){
        unfinishedPath_Paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        unfinishedPath_Paint.setColor(Color.GRAY);

        unfinishedCircle_Paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        unfinishedCircle_Paint.setColor(Color.GRAY); //getContext().getColor(R.color.SelcetedTransparent)); //Testing

        finished_Paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        finished_Paint.setColor(getContext().getColor(R.color.progress));

        pathWidth = 5;
        pathHeight = 5;
    }

    public void generatePath(int pathWidth, int pathHeight){
        this.pathWidth = pathWidth;
        this.pathHeight = pathHeight;
        if(pathHeight>=pathWidth) rectHeight = (height-MARGIN)/pathHeight/3;
        else rectHeight = (width-MARGIN)/pathWidth/3;


        baseRectangles = new Rect[pathHeight];

        circleRadius = rectHeight;
        circles = new boolean[pathWidth*pathHeight];
        for(int i = 0; i<circles.length;i++){
            circles[i] = false;
        }

        //background grey rectangles
        for(int i = 0; i<baseRectangles.length;i++){
            Rect baseRect = new Rect();
            baseRect.top = MARGIN + i*3 * rectHeight;
            baseRect.right = width-MARGIN;
            baseRect.left = MARGIN;
            baseRect.bottom = MARGIN + i*3 * rectHeight + rectHeight;
            baseRectangles[i] = baseRect;
        }

        //corner rectangles.
        if(pathHeight>0) {
            cornerRectangles = new Rect[pathHeight - 1];

            for (int i = 0; i < cornerRectangles.length; i++) {
                Rect cornerRect = new Rect();
                cornerRect.top = MARGIN + i * 3 * rectHeight + rectHeight;
                cornerRect.bottom = MARGIN + i * 3 * rectHeight + rectHeight * 3;

                if (i % 2 == 0) {
                    cornerRect.right = width - MARGIN - circleRadius / 4;
                    cornerRect.left = width - MARGIN - circleRadius / 4 - rectHeight;
                } else {
                    cornerRect.right = MARGIN + rectHeight + circleRadius / 4;
                    cornerRect.left = MARGIN + circleRadius / 4;
                }
                cornerRectangles[i] = cornerRect;

            }
        }

        postInvalidate();
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        width = w;
        height = h;
        super.onSizeChanged(w, h, oldw, oldh);
        generatePath(pathWidth,pathHeight);
//        updateProgress(0,100,1,0); //Just testing
    }

    @Override
    protected void onDraw(Canvas canvas) {
        canvas.drawColor(getContext().getColor(R.color.automaticVis_background)); //sets the background color of the canvas

        if(pathHeight>0) {//draws the corner rectangles.
            for (int i = 0; i < cornerRectangles.length; i++) {
                canvas.drawRect(cornerRectangles[i], unfinishedPath_Paint);
            }

            //draws the progress corner rectangles
            if(progressCornerRectangles != null){
                for(int i = 0; i < progressCornerRectangles.length;i++){
                    canvas.drawRect(progressCornerRectangles[i],finished_Paint);
                }
            }
        }

        //draws the background straight rectangles
        for(int y = 0; y<baseRectangles.length;y++){
            canvas.drawRect(baseRectangles[y],unfinishedPath_Paint);
        }

        if(progressRectangles != null && progressRectangles.length > 0){ //draws the progress
            for(int i = 0; i < progressRectangles.length; i++){
                canvas.drawRect(progressRectangles[i],finished_Paint);
            }
        }

        //draws the circles and the completed circles.
        for(int y = 0; y<baseRectangles.length;y++){
            for(int x = 0; x<pathWidth;x++){//draws the circles
                if (!circles[x+y*pathWidth]){
                    canvas.drawCircle(baseRectangles[y].left+circleRadius/4*3 + (baseRectangles[y].width()-2*(circleRadius/4*3))/(pathWidth-1)*x, baseRectangles[y].exactCenterY(), circleRadius,unfinishedCircle_Paint);
                } else{
                    canvas.drawCircle(baseRectangles[y].left+circleRadius/4*3 + (baseRectangles[y].width()-2*(circleRadius/4*3))/(pathWidth-1)*x, baseRectangles[y].exactCenterY(), circleRadius,finished_Paint);
                }
            }
        }
    }

    public void updateProgress(int seedsDropped, float percentage, int rowsCompleted, float percentageCorner){//W.I.P.

        if(circles != null){
            if(rowsCompleted%2==0) {
                for (int i = 0; i < rowsCompleted * pathWidth + seedsDropped; i++) {
                    if (circles.length > i) circles[i] = true;
                }
            } else{
                for (int i = 0; i < rowsCompleted * pathWidth; i++) {
                    if (circles.length > i) circles[i] = true;
                }
                for (int i = (rowsCompleted+1) * pathWidth-1; i > (rowsCompleted+1) * pathWidth-seedsDropped-1; i--) {
                    if (circles.length > i) circles[i] = true;
                }
            }
        }

        seedsDropped--;//because i'm lazy and don't feel to change this code

        if(seedsDropped>=pathWidth-1){ //again because i'm lazy, it's a quick fix to make sure the progress bar doesn't go off screen. it 00:11, i want to sleep oke.
            postInvalidate();
            return;
        }

        //rectangles
        progressRectangles = new Rect[rowsCompleted+1];
        if(rowsCompleted>0){
            for (int i = 0; i<progressRectangles.length-1;i++){
                Rect baseRect = new Rect();
                baseRect.top = MARGIN + i*3 * rectHeight;
                baseRect.right = width-MARGIN;
                baseRect.left = MARGIN;
                baseRect.bottom = MARGIN + i*3 * rectHeight + rectHeight;
                progressRectangles[i] = baseRect;
            }
        }

        //Progress rectangle
        float segmentWidth = (baseRectangles[progressRectangles.length-1].width()-2*(circleRadius/4*3))/(pathWidth-1);

        Rect baseRect = new Rect();
        baseRect.top = MARGIN + (progressRectangles.length-1)*3 * rectHeight;
        //                     |--------------Starting point (center first circle)-------------| |----------Amount to the right based on the percentage-----------|
        if(rowsCompleted%2==0) {
            baseRect.right = (int) (baseRectangles[progressRectangles.length-1].left+circleRadius/4*3+segmentWidth*(seedsDropped+1) - segmentWidth/100*(100-percentage));
            baseRect.left = MARGIN;
        }
        else {
            baseRect.left = (int) (baseRectangles[progressRectangles.length-1].right-circleRadius/4*3-segmentWidth*(seedsDropped+1) + segmentWidth/100*(100-percentage));
            baseRect.right = baseRectangles[progressRectangles.length-1].right;
        }
        baseRect.bottom = MARGIN + (progressRectangles.length-1)*3 * rectHeight + rectHeight;
        progressRectangles[progressRectangles.length-1] = baseRect;


        //corners
        if(rowsCompleted>0) {
            progressCornerRectangles = new Rect[rowsCompleted];

            if(progressCornerRectangles.length>1) {
                for (int i = 0; i < progressCornerRectangles.length-1; i++) {
                    Rect cornerRect = new Rect();
                    cornerRect.top = MARGIN + i * 3 * rectHeight + rectHeight;
                    cornerRect.bottom = MARGIN + i * 3 * rectHeight + rectHeight * 3;

                    if (i % 2 == 0) {
                        cornerRect.right = width - MARGIN - circleRadius / 4;
                        cornerRect.left = width - MARGIN - circleRadius / 4 - rectHeight;
                    } else {
                        cornerRect.right = MARGIN + rectHeight + circleRadius / 4;
                        cornerRect.left = MARGIN + circleRadius / 4;
                    }
                    progressCornerRectangles[i] = cornerRect;
                }
            }
            //progress corner.
            Rect cornerRect = new Rect();
            cornerRect.top = MARGIN + (progressCornerRectangles.length-1) * 3 * rectHeight + rectHeight;
            cornerRect.bottom = (int) (MARGIN + (progressCornerRectangles.length-1) * 3 * rectHeight + rectHeight * 3 * percentageCorner/100);

            if ((progressCornerRectangles.length-1) % 2 == 0) {
                cornerRect.right = width - MARGIN - circleRadius / 4;
                cornerRect.left = width - MARGIN - circleRadius / 4 - rectHeight;
            } else {
                cornerRect.right = MARGIN + rectHeight + circleRadius / 4;
                cornerRect.left = MARGIN + circleRadius / 4;
            }
            progressCornerRectangles[progressCornerRectangles.length-1] = cornerRect;


        }

        postInvalidate();
    }

    public void reset(){
        progressRectangles = null;
        progressCornerRectangles = null;

        for(int i = 0; i<circles.length;i++){
            circles[i] = false;
        }

        postInvalidate();
    }
}
